from flask import Blueprint, request, jsonify, make_response, json
from app.zone.models import Zone
from app.zone.schemas import zone_schema
from app import db
from app.utils import token_required, paginated_query, delete_record, single_record_query

zones = Blueprint('zones', __name__)
@zones.route('/api/v1/add/zone', methods=['POST'])
@token_required(1, 2, 3)
def create_zone(current_user):
    try:
        zone_dict = json.loads(request.get_json()['formData'])
        db.session.add(Zone(**zone_schema.load({
            'site_id': request.args.get('site_id'),
            'zone_name': zone_dict['zone_name'].strip(),
            'x_min': zone_dict['x_min'].strip(),
            'y_min': zone_dict['y_min'].strip(),
            'x_max': zone_dict['x_max'].strip(),
            'y_max': zone_dict['y_max'].strip()
        })))
        db.session.commit()
        return make_response(jsonify({'message': 'Zone added successfully', "status": "Success"}), 200)
    except Exception as e:
        return make_response(jsonify({'args': e.args, "message": 'failed added Zone', "status": "error"}), 500)

@zones.route('/api/v1/get/zone', methods=['GET'])
@token_required(1, 2, 3)
def get_zones(current_user):
    try:
        sites_paginated, data = paginated_query(Zone.query.order_by(Zone.zone_name.asc()),
            [Zone.zone_name],
                        Zone.site_id)
        if sites_paginated:
            serialized_zones = []
            for zone in sites_paginated:
                if zone.zone_id is 1:
                    continue
                serialized_zones.append({
                    'id': zone.zone_id,
                    'site_id': zone.site_id,
                    'zone_name': zone.zone_name,
                    'x_min': zone.x_min,
                    'y_min': zone.y_min,
                    'x_max': zone.x_max,
                    'y_max': zone.y_max
                })
            return jsonify({'zone': serialized_zones, "data": data}), 200
        return make_response(jsonify({'message': 'No zones found.', "data": data, "status": "Success"}), 200)
    except Exception as e:
        return jsonify({'message': f'Error: {e}', "status": "error"}), 500

@zones.route('/api/v1/get/zone_by_id/<int:zone_id>', methods=['GET'])
@token_required(1, 2, 3)
def get_zone_by_id(current_user, zone_id):
    try:
        zone = single_record_query(Zone, Zone.zone_id, zone_id, current_user)
        if zone:
            return make_response(jsonify({'zone': {
                'id': zone.zone_id,
                'site_id': zone.site_id,
                'zone_name': zone.zone_name,
                'x_min': zone.x_min,
                'y_min': zone.y_min,
                'x_max': zone.x_max,
                'y_max': zone.y_max,
            }}), 200)
        return make_response(jsonify({'message': 'Zone not found', "status": "error"}), 500)
    except Exception as e:
        return make_response(jsonify({'message': f'error: {e}', "status": "error"}), 500)

@zones.route('/api/v1/update/zone/<int:zone_id>', methods=['PUT'])
@token_required(1, 2, 3)
def update_zone(current_user, zone_id):
    try:
        zone_dict = json.loads(request.get_json()['formData'])
        zone = single_record_query(Zone, Zone.zone_id, zone_id, current_user)
        if not zone:
            return make_response(jsonify({'message': 'Zone not found', "status": "error"}), 500)
        if 'site_id' in zone_dict:
            zone.site_id = zone_dict['site_id']
        if 'zone_name' in zone_dict:
            zone.zone_name = zone_dict['zone_name'].strip()
        if 'x_min' in zone_dict:
            zone.x_min = zone_dict['x_min'].strip()
        if 'y_min' in zone_dict:
            zone.y_min = zone_dict['y_min'].strip()
        if 'x_max' in zone_dict:
            zone.x_max = zone_dict['x_max'].strip()
        if 'y_max' in zone_dict:
            zone.y_max = zone_dict['y_max'].strip()
        zone_schema.load(zone_schema.dump(zone), partial=True)
        db.session.commit()
        return make_response(jsonify({'message': 'Zone updated successfully', "status": "Success"}), 200)
    except Exception as e:
        return make_response(jsonify({'args': e.args, "message": 'Failed to update Zone', "status": "error"}), 500)

@zones.route('/api/v1/delete/zone/<int:zone_id>', methods=['DELETE'])
@token_required(1, 2, 3)
def delete_zone(current_user, zone_id):
    return delete_record(Zone, current_user, zone_id, "Zone", "zone_id")

@zones.route('/zone_name', methods=['GET'])
@token_required(1, 2, 3)
def zone_name(current_user):
    try:
        search = request.args.get('search')
        query = Zone.query.filter(Zone.site_id == request.args.get('site_id'))
        if search:
            query = query.filter(Zone.zone_name.ilike(f'%{search}%'))
        serialized_zones = []
        for zone in query:
            if zone.zone_id != 1:
                serialized_zones.append({
                    'id': zone.zone_id,
                    'name': zone.zone_name,
                    'site_id': zone.site_id
                })
        if serialized_zones:
            return jsonify({"zone": serialized_zones}), 200
        else:
            return jsonify({"zone": serialized_zones}), 200
    except Exception as e:
        return make_response(jsonify({"error": str(e)}), 500)